%* Prototype macro CutList by Jim Groeneveld, vs. 0.0.1. 29jul2010, 29may2012;
%* Purpose: remove specific elements from a list in a macro variable;
%* 0.0.1: return MacList if CutList is empty;

%MACRO CutMList (MacList=, CutList=, Sep=%STR( ), Returned=, Debug=0);
%* Returned needs a required argument when processing for macro code;

  %LOCAL MacName I J Err Warn Element Remove;
  %LET MacName  = CutList;
  %LET Err      = %SYSFUNC(COMPRESS(E R R O R, %STR( )));
  %LET Warn     = %SYSFUNC(COMPRESS(W A R N I N G, %STR( )));

  %IF (NOT %LENGTH(&MacList)) %THEN %DO;
    %PUT *** &MacName *** NOTE: No MacList of values specified, nothing to do;
    %GOTO Finish;
  %END;
  %ELSE %IF (NOT %LENGTH(&CutList)) %THEN %DO;
    %PUT *** &MacName *** NOTE: No CutList of values specified, nothing to do;
    &MacList /* to return - no semicolon - only if called from SAS code*/
    %GOTO Finish;
  %END;
%* Check;
  %IF (%UPCASE(&Returned) EQ RETURNED) %THEN %DO;
    %PUT *** &MacName *** WARNING: Returned argument may not be 'Returned', can not return this!;
    %GOTO Finish;
  %END;
  %ELSE %IF (%UPCASE(&Returned) NE NewList) %THEN %DO;
    %LOCAL Newlist /* to return */;
  %END;

  %LET I = 0; %LET NewList = ;
%if (&debug) %then %put &MacName ---> I=&I MacList=&MacList;
  %LET I = %EVAL(&I+1);
  %LET Element = %SCAN ( &MacList, &I, &Sep );
%if (&debug) %then %put &MacName ---> I=&I current Element=&Element;
  %DO %WHILE (%LENGTH(&Element));
    %LET J = 1;
    %LET Remove = %SCAN ( &CutList, &J, &Sep );
%if (&debug) %then %put &MacName ---> J=&J current Remove=&Remove;
    %DO %WHILE (%LENGTH(&Remove));
      %IF (%UPCASE(&Remove) EQ %UPCASE(&Element)) %THEN %DO;
        %LET I = %EVAL(&I+1);
        %LET Element = %SCAN ( &MacList, &I, &Sep );
%if (&debug) %then %put &MacName ---> I=&I removed Element=&Element;
      %END;
      %LET J = %EVAL (&J+1);
      %LET Remove = %SCAN ( &CutList, &J, &Sep );
%if (&debug) %then %put &MacName ---> J=&J next Remove=&Remove;
    %END;
    %IF (%LENGTH(&Element)) %THEN %LET NewList = &NewList &Element;
    %* else goto Finish;
%if (&debug) %then %put &MacName ---> I=&I J=&J NewList=&Newlist;
    %LET I = %EVAL(&I+1);
    %LET Element = %SCAN ( &MacList, &I, &Sep );
%if (&debug) %then %put &MacName ---> I=&I next Element=&Element;
  %END;
%if (&debug) %then %put &MacName ===> I=&I J=&J NewList=&Newlist;
  %IF (%LENGTH(&Returned)) %THEN %LET &Returned = &NewList; %* called from macro code;
  %ELSE &NewList /* no semicolon - only if called from SAS code*/

%Finish:
%MEND CutMList;

/* examples - - - - - - - - -
%put CutMList ===> NewList = %CutMList(MacList=, CutList=a b);
%put CutMList ===> NewList = %CutMList(MacList=a b, CutList=);
%put CutMList ===> NewList = %CutMList(MacList=a b c, CutList=a b);
%put CutMList ===> NewList = %CutMList(MacList=a b c, CutList=a b c);
%put CutMList ===> NewList = %CutMList(MacList=LISTING AUTO, CutList=AUTO, Debug=1);
%put CutMList ===> NewList = %CutMList(MacList=AUTO, CutList=AUTO, Debug=1);
%put CutMList ===> NewList = %CutMList(MacList=LISTING AUTO third, CutList=AUTO, Debug=1);
%put CutMList ===> NewList = %CutMList(MacList=AUTO, CutList=AUTO, Debug=1);
%put CutMList ===> NewList = %CutMList(MacList=AUTO HTML PDF, CutList=AUTO, Debug=1);

%GLOBAL Trimmed;
%CutMList(MacList=AUTO HTML PDF, CutList=AUTO, Returned=Trimmed, Debug=1);
%put CutMList ===> Trimmed = &Trimmed;
- - - - - - - - - */
